SystemOrganization addCategory: #'Collections-BTree'!

TestCase subclass: #BTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 17:48'!
assertConsistent: aBTree
	| depths root |
	root _ aBTree root.
	
 " http://www.cs.umbc.edu/~woodcock/cmsc341/btree/defn.html

A B-Tree of order m is a search tree, where the data 
  (or pointers to it) is stored at the leaves, such that:

1) the root is either a leaf (as well--i.e. the tree is 
     a single node) or has between 2 and m children"
	
	self assert: (root isLeaf or: [root values size between: 2 and: aBTree order]).
	
"  2) all other (non-root) nodes have at least m/2 children,
     but no more than m children"

	root allChildrenDo: [:ea | self assert: (ea values size between: (aBTree order // 2) and: aBTree order)]. 

"  3) all leaves are at the same depth"

	depths _ Set new.
	root withAllChildrenDo: [:ea | ea isLeaf ifTrue: [depths add: ea depth]].
	self assert: depths size = 1.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 22:48'!
assertSorted: aTree
	| last |
	last _ nil.
	aTree keysAndValuesDo:
		[:k :v |
		last ifNotNil: [self assert: last < k].
		last _ k]! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:59'!
checkTreeWithOrder: order items: count
	self checkTree: (BTree order: order) items: count! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:58'!
checkTree: tree array: pairArray
	pairArray withIndexDo:
		[:pair :i |
		tree at: pair first put: pair second.
		self assert: tree size = i.
		"self assertConsistent: tree.
		self assertSorted: tree"].
	pairArray shuffled do:
		[:pair |
		self assert: (tree at: pair first) = pair second].
	self assert: tree size = pairArray size.
	self assertConsistent: tree.
	self assertSorted: tree.
	^ tree
	
	 ! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:59'!
checkTree: tree items: count
	| pairArray  |
	pairArray _ ((1 to: count) collect: [:i | Array with: i asString with: i]) shuffled.
	self checkTree: tree array: pairArray! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 18:08'!
testAddAndRetrieveWithSplit
	self checkTreeWithOrder: 4 items: 5! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 18:08'!
testAddAndRetrieveWithTwoSplits
	self checkTreeWithOrder: 4 items: 17! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 4/20/2005 14:45'!
testCommonKeysDifferentValues
	| large small expectedResults results |
	large _ BTree new.
	1 to: 1000 by: 2 do: [:i | large at: i put: i].
	small _ BTree new.
	2 to: 2000 by: 11 do: [:i | small at: i put: i*2].
	
	expectedResults _ ((-9 to: 1000 by: 22) allButFirst collect: [:i | Array with: i with: i with: i*2]) asOrderedCollection.
	
	results _ OrderedCollection new.
	large commonKeysWith: small keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v1 with: v2)].
	self assert: results = expectedResults.
	
	results _ OrderedCollection new.
	small commonKeysWith: large keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v2 with: v1)].
	self assert: results = expectedResults! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 4/20/2005 14:45'!
testCommonKeysSameValues
	| large small expectedResults results |
	large _ BTree new.
	1 to: 1000 by: 2 do: [:i | large at: i put: i].
	small _ BTree new.
	2 to: 2000 by: 11 do: [:i | small at: i put: i].
	
	expectedResults _ ((-9 to: 1000 by: 22) allButFirst collect: [:i | Array with: i with: i with: i]) asOrderedCollection.
	
	results _ OrderedCollection new.
	large commonKeysWith: small keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v1 with: v2)].
	self assert: results = expectedResults.
	
	results _ OrderedCollection new.
	small commonKeysWith: large keysAndValuesDo:
		[:k :v1 :v2 |
		results add: (Array with: k with: v2 with: v1)].
	self assert: results = expectedResults! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 02:06'!
testEmptyTree
	self checkTreeWithOrder: 5 items: 0! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:44'!
testLargeTree
	"self checkTreeWithOrder: 8 items: 5000"
	
! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:08'!
testMissing
	| tree |
	tree _ BTree new.
	tree at: 1 put: 'foo'.
	tree at: 2 put: 'bar'.
	self assert: (tree at: 1 ifAbsent: [42]) = 'foo'.
	self assert: (tree at: 2 ifAbsent: [42]) = 'bar'.
	self assert: (tree at: 3 ifAbsent: [42]) = 42.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 02:06'!
testMultipleAddAndRetrieve
	self checkTreeWithOrder: 5 items: 3! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 10/8/2004 18:06'!
testRange
	| tree keys |
	tree _ BTree new.
	1 to: 100 by: 2 do: [:ea | tree at: ea put: ea * 2].
	self assert: tree size = 50.
	keys _ OrderedCollection new.
	tree from: 12 to: 19 keysAndValuesDo:
		[:k :v |
		self assert: k * 2 = v.
		keys add: k].
	self assert: keys asArray = #(13 15 17 19).
	self assertSorted: tree.
	self assertConsistent: tree.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 18:08'!
testRemoval
	| tree |
	tree _ BTree new.
	#(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
	self assert: tree size = 12.
	tree removeKey: #d.
	self assert: tree size = 11.
	self assert: (tree at: #d ifAbsent: [32]) = 32.
	tree removeKey: #k.
	self assert: tree size = 10.
	self assert: (tree at: #k ifAbsent: [32]) = 32.
	self assertSorted: tree.
	self assertConsistent: tree.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 1/11/2005 13:12'!
testRemoval2
	| tree |
	tree := BTree new.
	1 to: 15 do: [:i | tree at: i put: i].
	1 to: 15 do: [:i | tree removeKey: i].
	self assert: tree isEmpty! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:06'!
testRemoveAll
	| tree |
	tree _ BTree new.
	#(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
	self assert: tree size = 12.
	#(a b c d e f g h i j k l)  shuffled do: [:ea | tree removeKey: ea].
	self assert: tree size = 0.

	self assertSorted: tree.
	self assertConsistent: tree.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:56'!
testRemoveAllFailing
	| tree |
	tree _ BTree new.
	#(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
	self assert: tree size = 12.
	#(#c #j #g #i #k #l #f #e #d #h #a #b) do: [:ea | tree removeKey: ea].
	self assert: tree size = 0.

	self assertSorted: tree.
	self assertConsistent: tree.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:07'!
testRemoveLots
	| tree |
	tree _ BTree new.
	#(a b c d e f g h i j k l) do: [:ea | tree at: ea put: ea, '!!'].
	self assert: tree size = 12.
	#(a c e g i k) shuffled do: [:ea | tree removeKey: ea].
	self assert: tree size = 6.

	self assertSorted: tree.
	self assertConsistent: tree.! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 02:06'!
testSimpleAddAndRetrieve
	self checkTreeWithOrder: 5 items: 1! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 22:01'!
testStoreNil
	self checkTree: BTree new array: {{'x'. 1}. {'y'. nil}}! !

!BTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:59'!
testStringKeys
	self checkTree: (BTree keys: BTreeStringKeys new) items: 100! !

TestCase subclass: #TSTreeTest
	instanceVariableNames: 'tree empty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 9/21/2004 20:38'!
setUp
	tree := TSTree new.
	self words shuffled do: [:ea | tree at: ea put: ea asUppercase].
	empty _ TSTree new.! !

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 9/22/2004 01:01'!
testAt
	self assert: (tree at: 'abear') = 'ABEAR'.
	self assert: (tree at: 'abelmosk') = 'ABELMOSK'.
	self assert: (tree at: 'abracadabra' ifAbsent: [42]) = 42.
	self assert: (tree at: '' ifAbsent: [42]) = 42.
	self assert: (empty at: 'foo' ifAbsent: [42]) = 42 ! !

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 9/21/2004 20:38'!
testMatch
	self assert: (tree matchesForString: 'abea' distance: 1) asSet = #(ABEAM ABEAR ABED ABEL ABET ABE) asSet.
	self assert: (tree matchesForString: 'abel' distance: 2) asSet = #(ABELIA ABEAM ABEAR) asSet.
	self assert: (tree matchesForString: 'abel' distance: 0) = #(ABEL).
	self assert: (tree matchesForString: '' distance: 1) = #().
	self assert: (empty matchesForString: 'foo' distance: 0) = #()! !

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 9/22/2004 00:48'!
testPrefix
	self assert: (tree matchesForPrefix: 'abet') asSet = #(ABET ABETMENT) asSet.
	self assert: (tree matchesForPrefix: 'abelm') asSet = #(ABELMOSCHUS ABELMOSK) asSet.
	self assert: (tree matchesForPrefix: '') asSet = (self words asSet collect: [:ea | ea asUppercase]).
	self assert: (empty matchesForPrefix: 'foo') = #()! !

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 12/13/2004 13:07'!
testRemove
	self assert: (tree at: 'abel') = 'ABEL'.
	self assert: (tree removeKey: 'abel') = 'ABEL'.
	self assert: (tree at: 'abel' ifAbsent: [42]) = 42.
	self assert: (tree at: 'abelmosk') = 'ABELMOSK'.
	self assert: (tree removeKey: 'foo' ifAbsent: [42]) = 42.
	self assert: (tree removeKey: 'abel' ifAbsent: [42]) = 42.! !

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 9/21/2004 20:38'!
testValues
	self assert: tree values asSet = (self words asSet collect: [:ea | ea asUppercase]).
	self assert: empty values isEmpty! !

!TSTreeTest methodsFor: 'as yet unclassified' stamp: 'avi 9/22/2004 00:58'!
words
	^ #(
abe
abeam
abear
abearance
abecedarian
abecedarium
abecedary
abed
abeigh
abel
abele
abelia
abelian
abelicea
abelite
abelite
abelmoschus
abelmosk
abelonian
abeltree
abencerrages
abenteric
abepithymia
aberdeen
aberdevine
aberdonian
aberia
aberrance
aberrancy
aberrant
aberrate
aberration
aberrational
aberrator
aberrometer
aberroscope
aberuncator
abet
abetment) ! !

Object subclass: #BTreeKeys
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

BTreeKeys variableSubclass: #BTreeKeysArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 20:40'!
canGrow
	^ (self at: self size) isNil! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 18:00'!
canShrink
	^  (self at: self size // 2 + 1) notNil! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/25/2004 23:39'!
emptyCopy
	^ self class new: self size! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:39'!
findIndexForKey: aMagnitude
	self withIndexDo:
		[:key :i |
		(key isNil or: [key > aMagnitude]) ifTrue:
			[^ i - 1]].
	^ self size
! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:02'!
first
	^ self at: 1! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 17:40'!
shiftLeftTo: index
	index to: self size - 1 by: 1 do:
		[:i |
		self at: i put: (self at: i+1)].
	self at: self size put: nil.! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/25/2004 23:37'!
shiftRightFrom: index
	self size to: index+1 by: -1 do:
		[:i |
		self at: i put: (self at: i - 1)]
	! !

!BTreeKeys methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:02'!
withIndexDo: aBlock
	1 to: self size do: [:i | aBlock value: (self at: i) value: i]! !

BTreeKeys subclass: #BTreeStringKeys
	instanceVariableNames: 'keys prefix abbreviations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!BTreeStringKeys class methodsFor: 'as yet unclassified' stamp: 'avi 2/25/2004 14:31'!
new
	^ self new: 8! !

!BTreeStringKeys class methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:04'!
new: aNumber
	^ self basicNew initializeWithSize: aNumber! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:00'!
abbreviationSize
	^ 3! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:43'!
abbreviationsAndIndicesDo: aBlock
	| stream |
	stream _ abbreviations readStream.
	1 to: self size do:
		[:i |
		stream atEnd
			ifFalse: [aBlock value: prefix, (stream next: self abbreviationSize) value: i]
			ifTrue: [aBlock value: nil value: i]]! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 15:41'!
at: aNumber
	^ keys at: aNumber! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:51'!
at: aNumber put: aString
	keys at: aNumber put: aString.
	prefix _ self nilPrefix.! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:39'!
buildAbbreviationsFrom: readStreams
	| nextChars |
	1 to: self abbreviationSize do:
		[:i |
		nextChars _ readStreams collect: [:ea | ea next ifNil: [Character value: 0]].
		nextChars withIndexDo:
			[:c :j |
			abbreviations at: (j-1 * self abbreviationSize) + i put: c]].
	^ abbreviations! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:42'!
extractPrefixFrom: readStreams
	| prefixStream nextChars |
	prefixStream _ String new writeStream.
	
	[readStreams anySatisfy: [:ea | ea atEnd]] whileFalse:
		[nextChars _ readStreams collect: [:ea | ea next].
		(nextChars allSatisfy: [:ea | ea = nextChars first])
				ifTrue: [prefixStream nextPut: nextChars first]
				ifFalse: [readStreams do: [:ea | ea skip: -1]. ^ prefixStream contents]].
	^ prefixStream contents
! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:51'!
findIndexForKey: aString
	| stream str diff |
	prefix = self nilPrefix ifTrue: [self rebuildAbbreviations].
	stream _ aString readStream.
	str _ stream next: prefix size + self abbreviationSize.
	diff _ prefix size + self abbreviationSize - str size.
	str _ str, (String new: diff).
	self abbreviationsAndIndicesDo:
		[:abbr :i |
		abbr ifNil: [^ i - 1].
		str < abbr ifTrue: [^ i - 1].
		str = abbr ifTrue: [^ super findIndexForKey: aString]].
	^ self size! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:51'!
initializeWithSize: aNumber
	keys _ Array new: aNumber.
	prefix _ self nilPrefix.! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:51'!
nilPrefix
	^ '^^^'! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 18:47'!
rebuildAbbreviations
	| keyStreams filled |
	filled _ keys count: [:ea | ea notNil].
	abbreviations _ String new: (filled * self abbreviationSize).
	filled = 0 ifTrue: [prefix _ ''. ^ self ].
	keyStreams _ (1 to: filled) collect: [:i | (keys at: i) readStream].
	
	prefix _ self extractPrefixFrom: keyStreams.
	abbreviations _ self buildAbbreviationsFrom: keyStreams.
! !

!BTreeStringKeys methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 16:47'!
size
	^ keys size! !

Object variableSubclass: #BTreeNode
	instanceVariableNames: 'parent keys'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

BTreeNode variableSubclass: #BTreeInteriorNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:40'!
childForKey: aMagnitude
	| index |
	index _ keys findIndexForKey: aMagnitude.
	index = 0 ifTrue:
		[keys at: 1 put: aMagnitude.
		^ self at: 1].
	^ self at: index
		
! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/23/2004 21:47'!
childrenDo: aBlock
	self valuesDo: aBlock! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 4/20/2005 17:20'!
commonKeysWith: aNode keysAndValuesDo: aBlock flip: aBoolean
	| index |
	aNode firstKey < self firstKey ifTrue: [^ aNode commonKeysWith: self keysAndValuesDo: aBlock flip: aBoolean not].
	index _ (keys findIndexForKey: aNode firstKey) max: 1.
	index to: self size do:
		[:i |
		(self at: i) ifNotNilDo: [:c | c commonKeysWith: aNode keysAndValuesDo: aBlock flip: aBoolean]]! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 4/21/2005 02:28'!
depth
	^ 1 + self firstChild depth! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'jcg 1/17/2005 22:59'!
existingChildForKey: aMagnitude
       "Unlike #childForKey:, this method looks for a child, but doesn't mess with the tree if it doesn't exist."
       | index |
       index _ keys findIndexForKey: aMagnitude.
       index = 0
               ifTrue: [^ nil]
               ifFalse: [^ self at: index].! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'jcg 1/17/2005 23:00'!
existingLeafForKey: aMagnitude
       "Unlike #leafForKey:, this method looks for a leaf but doesn't mess with the tree if it doesn't exist."
       | child |
       child _ self existingChildForKey: aMagnitude.
       ^ child ifNotNil: [child existingLeafForKey: aMagnitude]! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 00:39'!
firstChild
	self childrenDo: [:ea | ^ ea].
	self error: 'No children'.! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 12:05'!
insertKey: aMagnitude value: anObject at: index
	super insertKey: aMagnitude value: anObject at: index.
	anObject parent: self! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 00:22'!
isLeaf
	^ false! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/24/2004 01:59'!
leafForKey: aMagnitude
	^ (self childForKey: aMagnitude) leafForKey: aMagnitude! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 10/8/2004 18:04'!
leavesFrom: start to: end do: aBlock
	| startIndex endIndex |
	startIndex _ (keys findIndexForKey: start) max: 1.
	endIndex _ (keys findIndexForKey: end).
	startIndex to: endIndex do: [:i | (self at: i) leavesFrom: start to: end do: aBlock]! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 2/27/2004 21:01'!
nextSiblingForChild: aNode
	| index |
	index _ keys findIndexForKey: aNode firstKey.
	^ (index = self size or: [(keys at: index+1) isNil]) 
		ifTrue: [index = 1 ifFalse: [self at: index - 1] ifTrue: [nil]]
		ifFalse: [self at: index + 1]! !

!BTreeInteriorNode methodsFor: 'as yet unclassified' stamp: 'avi 1/11/2005 13:13'!
updateKey: oldMagnitude to: newMagnitude
       keys withIndexDo:
               [:key :i |
               key = oldMagnitude ifTrue:
                       [(i = 1 and: [parent notNil]) ifTrue:
                               [parent updateKey: oldMagnitude to: newMagnitude].
                       ^ keys at: i put: newMagnitude]].
       self error: 'No such key'! !

BTreeNode variableSubclass: #BTreeLeafNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 2/23/2004 21:45'!
childrenDo: aBlock
	"no children"! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'ac 5/17/2005 15:01'!
commonKeysWith: aNode keysAndValuesDo: aBlock flip: aBoolean
	| index key block leaf advanceKey last |

	aNode firstKey ifNil: [^ self].

	block _ aBoolean ifTrue: [[:k :v1 :v2 | aBlock value: k value: v2 value: v1]] ifFalse: [aBlock].

	index _ 0.
	advanceKey _
		[index _ index + 1.
		index > self size ifTrue: [^ self].
		key _ keys at: index.
		key ifNil: [^ self]].
	last _ self lastKey.
	
	advanceKey value.
	[key < aNode firstKey] whileTrue: advanceKey.
		
	[leaf _ aNode existingLeafForKey: key.
	leaf lastKey < key ifTrue: advanceKey ifFalse:
		[leaf keysAndValuesDo:
			[:otherKey :otherValue |
			otherKey > last ifTrue: [^ self].
			[key < otherKey] whileTrue: advanceKey.
			key = otherKey ifTrue: [block value: key value: (self at: index) value: otherValue]].
		key > leaf lastKey ifFalse: advanceKey]] repeat
! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 4/21/2005 02:28'!
depth
	^ 1! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'jcg 1/17/2005 23:08'!
existingLeafForKey: aMagnitude
       ^ self! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 2/23/2004 21:22'!
isLeaf
	^ true! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 4/20/2005 16:41'!
keys
	^ Array streamContents: [:s | self keysDo: [:ea | s nextPut: ea]]! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 4/20/2005 16:34'!
lastKey
	| last |
	last _ nil.
	self keysDo: [:k | last _ k].
	^ last! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 2/23/2004 21:41'!
leafForKey: aMagnitude
	^ self! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 10/8/2004 18:00'!
leavesFrom: start to: end do: aBlock
	aBlock value: self! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:40'!
valueForKey: aMagnitude ifAbsent: errorBlock
	| i |
	i _ keys findIndexForKey: aMagnitude.
	^ (i > 0 and: [(keys at: i) = aMagnitude])
		ifTrue: [self at: i]
		ifFalse: [errorBlock value]
! !

!BTreeLeafNode methodsFor: 'as yet unclassified' stamp: 'avi 4/20/2005 15:03'!
valueForKey: aMagnitude ifPresent: aBlock
	^ aBlock value: (self valueForKey: aMagnitude ifAbsent: [^ nil])! !

!BTreeNode class methodsFor: 'as yet unclassified' stamp: 'avi 2/25/2004 12:28'!
keys: anArray
	^ (self new: (anArray size)) keys: anArray! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 2/23/2004 21:24'!
allChildrenDo: aBlock
	self childrenDo:
		[:ea |
		aBlock value: ea.
		ea allChildrenDo: aBlock]! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 2/23/2004 21:34'!
allLeavesDo: aBlock
	self withAllChildrenDo: [:ea | ea isLeaf ifTrue: [aBlock value: ea]]! !

!BTreeNode methodsFor: 'testing' stamp: 'avi 2/27/2004 20:41'!
canGrow
	^ keys canGrow! !

!BTreeNode methodsFor: 'testing' stamp: 'avi 2/26/2004 17:59'!
canShrink
	^ keys canShrink! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/23/2004 21:23'!
children
	^ Array streamContents: [:s | self childrenDo: [:ea | s nextPut: ea]]! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 2/23/2004 21:44'!
childrenDo: aBlock
	self subclassResponsibility! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/23/2004 21:27'!
depth
	^ parent ifNil: [1] ifNotNil: [1 + parent depth]! !

!BTreeNode methodsFor: 'private' stamp: 'avi 4/29/2005 12:57'!
ensureParent
	parent ifNil:
		 [self parent: (self interiorNodeClass keys: keys emptyCopy).
		parent insertKey: self firstKey value: self].
	^ parent! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/26/2004 00:35'!
firstKey
	^ keys first! !

!BTreeNode methodsFor: 'private' stamp: 'avi 4/29/2005 12:57'!
grow
	| sibling |
	parent ifNotNil:
		[sibling _ parent nextSiblingForChild: self.
		sibling ifNil: ["we're the new root". self parent: nil. ^ self].
		sibling canShrink
			ifTrue: [self stealFrom: sibling]
			ifFalse: [self mergeWith: sibling]]
				! !

!BTreeNode methodsFor: 'inserting' stamp: 'avi 2/27/2004 20:41'!
insertKey: aMagnitude value: anObject
	| index key |
	index _ keys findIndexForKey: aMagnitude.
	index = 0 ifTrue:
		[self canGrow
			ifTrue:
				[self shiftRightFrom: 1.
				^ self insertKey: aMagnitude value: anObject at: 1]
			ifFalse:
				[self split.
				^ (parent childForKey: aMagnitude) insertKey: aMagnitude value: anObject]].
	
	key _ keys at: index.
	key = aMagnitude ifTrue:
		[^ self insertKey: aMagnitude value: anObject at: index].
	index < self size ifTrue:
		[key _ keys at: index + 1.
		key
			ifNil: [^ self insertKey: aMagnitude value: anObject at: index+1]
			ifNotNil:
				[self canGrow ifTrue:
					[self shiftRightFrom: index+1.
					^ self insertKey: aMagnitude value: anObject at: index+1]]].

	"otherwise"
	self split.
	^ (parent childForKey: aMagnitude) insertKey: aMagnitude value: anObject! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/25/2004 23:29'!
insertKey: aMagnitude value: anObject at: index
	keys at: index put: aMagnitude.
	self at: index put: anObject! !

!BTreeNode methodsFor: 'private' stamp: 'avi 4/29/2005 12:51'!
interiorNodeClass
	^ BTreeInteriorNode! !

!BTreeNode methodsFor: 'testing' stamp: 'avi 2/24/2004 00:22'!
isLeaf
	self subclassResponsibility! !

!BTreeNode methodsFor: 'private' stamp: 'avi 5/1/2005 13:56'!
keyArray
	^ keys! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 2/25/2004 23:31'!
keysAndValuesDo: aBlock
	keys withIndexDo:
		[:key :i |
		key ifNotNil: [aBlock value: key value: (self at: i)]]! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 4/20/2005 16:33'!
keysDo: aBlock
	keys withIndexDo:
		[:key :i |
		key ifNotNil: [aBlock value: key] ifNil: [^ self]]! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/25/2004 12:27'!
keys: anArray
	keys _ anArray! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 10/8/2004 17:57'!
leavesFrom: start to: end do: aBlock
	self subclassResponsibility! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/27/2004 21:05'!
mergeWith: aNode		
	| oldKey |
	oldKey _ self firstKey.
	aNode keysAndValuesDo:
		[:k :v |
		self insertKey: k value: v].
	parent removeKey: aNode firstKey.
	parent updateKey: oldKey to: self firstKey.! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/24/2004 00:24'!
parent
	^ parent! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/24/2004 00:24'!
parent: aBTreeNode
	parent _ aBTreeNode! !

!BTreeNode methodsFor: 'removing' stamp: 'avi 2/27/2004 21:55'!
removeKey: aMagnitude
	| index key |
	self canShrink ifFalse: [self grow].
	
	index _ keys findIndexForKey: aMagnitude.
	key _ keys at: index.
	key = aMagnitude ifFalse: [^ self error: 'No such key'].
	
	self shiftLeftTo: index.
	
	index = 1 ifTrue: [parent ifNotNil: [parent updateKey: key to: self firstKey]]! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/23/2004 22:17'!
root
	^ parent
		ifNil: [self]
		ifNotNil: [parent root]! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/26/2004 17:39'!
shiftLeftTo: index
	keys shiftLeftTo: index.
	index to: self size - 1 by: 1 do:
		[:i |
		self at: i put: (self at: i+1)].
	self at: self size put: nil.! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/25/2004 23:37'!
shiftRightFrom: index
	keys shiftRightFrom: index.
	self size to: index+1 by: -1 do:
		[:i |
		self at: i put: (self at: i-1)]! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/25/2004 23:39'!
split
	| other midpoint |
	other _ self class keys: keys emptyCopy.
	midpoint _ self size // 2 + 1.
	midpoint to: self size do:
		[:i |
		other insertKey: (keys at: i) value: (self at: i) at: (i - midpoint + 1).
		keys at: i put: nil.
		self at: i put: nil].
	
	self ensureParent insertKey: other firstKey value: other! !

!BTreeNode methodsFor: 'private' stamp: 'avi 2/27/2004 21:55'!
stealFrom: aNode
	| key value |
	aNode firstKey > self firstKey
		ifTrue: [value _ aNode at: 1. key _ aNode firstKey]
		ifFalse:
			[aNode keysAndValuesDo: [:k :v | key _ k. value _ v].
			parent ifNotNil: [parent updateKey: self firstKey to: key]].
	self insertKey: key value: value.
	aNode removeKey: key! !

!BTreeNode methodsFor: 'accessing' stamp: 'avi 2/23/2004 21:47'!
values
	^ Array streamContents: [:s | self valuesDo: [:ea | s nextPut: ea]]! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 2/23/2004 21:59'!
valuesDo: aBlock
	self keysAndValuesDo: [:k :v | aBlock value: v]! !

!BTreeNode methodsFor: 'enumerating' stamp: 'avi 2/23/2004 21:27'!
withAllChildrenDo: aBlock
	aBlock value: self.
	self allChildrenDo: aBlock.! !

!Object methodsFor: '*collections-btree' stamp: 'avi 5/1/2005 14:10'!
isTreeSet
	^ false! !

Object subclass: #TSTreeNode
	instanceVariableNames: 'key value low high equal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!TSTreeNode class methodsFor: 'as yet unclassified' stamp: 'avi 9/22/2004 01:03'!
key: aCharacter
	^ self basicNew initializeWithKey: aCharacter! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 12/13/2004 13:02'!
canBeCulled
	^ self value isNil
		and: [low isNil]
		and: [equal isNil]
		and: [high isNil]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 12/13/2004 13:02'!
cullNode: aNode
	low == aNode ifTrue: [^ low _ nil].
	equal == aNode ifTrue: [^ equal _ nil].
	high == aNode ifTrue: [^ high _ nil]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:00'!
do: aBlock
	self nodesDo: [:ea | ea value ifNotNilDo: aBlock]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:03'!
initializeWithKey: aCharacter
	key _ aCharacter! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 12/13/2004 13:26'!
lookupString: aString startingAt: i
"inlined for performance"
"
	self
		lookupString: aString
		startingAt: i
		whenFound: [^ value]
		whenNil: [:c | ^ nil]
		recurseWith: [:node :j | ^ node lookupString: aString startingAt: j]"
	| char |
	char := aString at: i.
	char = key
		ifTrue:
			[aString size = i
				ifTrue: [^ value]
				ifFalse: [^ equal ifNotNil: [equal lookupString: aString startingAt: i+1]]]
		ifFalse:
			[char < key
				ifTrue: [^ low ifNotNil: [low lookupString: aString startingAt: i]]
				ifFalse: [^ high ifNotNil: [high lookupString: aString startingAt: i]]]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:41'!
lookupString: aString startingAt: i insert: anObject
	self
		lookupString: aString
		startingAt: i
		whenFound: [self value: anObject]
		whenNil: [:c | self newNodeWithKey: c]
		recurseWith: [:node :j | node lookupString: aString startingAt: j insert: anObject]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:04'!
lookupString: aString startingAt: i whenFound: foundBlock whenNil: nilBlock recurseWith: recurseBlock
	| char |
	char := aString at: i.
	char = key
		ifTrue:
			[aString size = i
				ifTrue: [foundBlock value]
				ifFalse: [equal ifNil: [equal := nilBlock value: (aString at: i+1)].
						 recurseBlock value: equal value: i+1]]
		ifFalse:
			[char < key
				ifTrue: [low ifNil: [low := nilBlock value: char].
						recurseBlock value: low value: i]
				ifFalse: [high ifNil: [high := nilBlock value: char].
						recurseBlock value: high value: i]]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:04'!
matchesForPrefix: aString startingAt: i do: aBlock
	self
		lookupString: aString
		startingAt: i
		whenFound: [value ifNotNil: [aBlock value: value].  equal ifNotNil: [equal do: aBlock]]
		whenNil: [:c | ^ self]
		recurseWith: [:n :j | n matchesForPrefix: aString startingAt: j do: aBlock]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 10/20/2004 21:27'!
matchesForString: aString startingAt: i distance: d do: aBlock nodesDo: nodeBlock
	
	| char d2 |
	nodeBlock value: self.
	d < 0 ifTrue: [^ self].
	
	char := aString at: i.
	(d > 0 or: [char < key])
		ifTrue: [low ifNotNil: [low matchesForString: aString startingAt: i distance: d do: aBlock nodesDo: nodeBlock]].
		
	d2 := char = key ifTrue: [d] ifFalse: [d-1].
	(i + d2 = aString size and: [value notNil]) ifTrue: [aBlock value: value].
	equal ifNotNil: [equal matchesForString: aString startingAt: (i+1 min: aString size) distance: d2 do: aBlock nodesDo: nodeBlock].
	
	(d > 0 or: [char > key])
		ifTrue: [high ifNotNil: [high matchesForString: aString startingAt: i distance: d do: aBlock nodesDo: nodeBlock]]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:41'!
newNodeWithKey: aCharacter
	^ self class key: aCharacter! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 00:51'!
nodesDo: aBlock
	aBlock value: self.
	low ifNotNil: [low nodesDo: aBlock].
	equal ifNotNil: [equal nodesDo: aBlock].
	high ifNotNil: [high nodesDo: aBlock]! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 12/13/2004 13:10'!
removeString: aString startingAt: i
	| val |
	self
		lookupString: aString
		startingAt: i
		whenFound: [val _ self value. self value: nil]
		whenNil: [:c | ^ nil]
		recurseWith:
			[:node :j |
			val _ node removeString: aString startingAt: j.
			node canBeCulled ifTrue:
				[self cullNode: node]].
	^ val! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 00:51'!
value
	^ value! !

!TSTreeNode methodsFor: 'private' stamp: 'avi 9/22/2004 01:41'!
value: anObject
	value _ anObject! !

Collection subclass: #BTree
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!BTree class methodsFor: 'as yet unclassified' stamp: 'avi 3/2/2004 15:03'!
keys: aBTreeKeys
	^ self basicNew initializeWithKeys: aBTreeKeys! !

!BTree class methodsFor: 'as yet unclassified' stamp: 'avi 2/25/2004 12:38'!
new
	^ self order: 5! !

!BTree class methodsFor: 'as yet unclassified' stamp: 'avi 2/26/2004 00:01'!
order: aNumber
	^ self keys: (BTreeKeysArray new: aNumber)! !

!BTree methodsFor: 'accessing' stamp: 'avi 2/23/2004 21:51'!
at: aMagnitude
	^ self at: aMagnitude ifAbsent: [self error: aMagnitude printString, ' not found']! !

!BTree methodsFor: 'accessing' stamp: 'jcg 1/17/2005 23:09'!
at: aMagnitude ifAbsent: errorBlock
       | leaf |
       leaf _ root existingLeafForKey: aMagnitude.
       leaf ifNil: [^ errorBlock value].
       ^ leaf valueForKey: aMagnitude ifAbsent: errorBlock! !

!BTree methodsFor: 'adding' stamp: 'avi 2/24/2004 12:08'!
at: aMagnitude put: anObject
	| leaf |
	leaf _ root leafForKey: aMagnitude.
	leaf insertKey: aMagnitude value: anObject.
	root _ leaf root.
	^ anObject! !

!BTree methodsFor: 'enumerating' stamp: 'avi 4/21/2005 02:41'!
commonKeysWith: aTree keysAndValuesDo: aBlock
	^ aTree depth < self depth
		ifTrue: [aTree root commonKeysWith: root keysAndValuesDo: aBlock flip: true]
		ifFalse: [root commonKeysWith: aTree root keysAndValuesDo: aBlock flip: false]! !

!BTree methodsFor: 'accessing' stamp: 'avi 4/21/2005 02:28'!
depth
	^ root depth! !

!BTree methodsFor: 'enumerating' stamp: 'avi 2/23/2004 21:49'!
do: aBlock
	root allLeavesDo: [:ea | ea valuesDo: aBlock]! !

!BTree methodsFor: 'copying' stamp: 'avi 5/1/2005 14:51'!
emptyCopy
	^ BTree keys: root keyArray emptyCopy! !

!BTree methodsFor: 'enumerating' stamp: 'avi 10/8/2004 18:04'!
from: start to: end do: aBlock
	self from: start to: end keysAndValuesDo: [:k :v | aBlock value: v]! !

!BTree methodsFor: 'enumerating' stamp: 'avi 10/8/2004 18:01'!
from: start to: end keysAndValuesDo: aBlock
	root leavesFrom: start to: end do:
		[:ea |
		ea keysAndValuesDo:
			[:k :v |
			(k between: start and: end) ifTrue:
				[aBlock value: k value: v]]]! !

!BTree methodsFor: 'initialize-release' stamp: 'avi 3/2/2004 15:02'!
initializeWithKeys: aBTreeKeys
	aBTreeKeys size > 3 ifFalse: [self error: 'The BTree order must be at least 4'].
	root _ BTreeLeafNode keys: aBTreeKeys! !

!BTree methodsFor: 'accessing' stamp: 'avi 4/20/2005 14:29'!
keys
	^ Array streamContents:
		[:s |
		self keysDo: [:k | s nextPut: k]]! !

!BTree methodsFor: 'enumerating' stamp: 'avi 2/24/2004 22:49'!
keysAndValuesDo: aBlock
	root allLeavesDo: [:ea | ea keysAndValuesDo: aBlock]! !

!BTree methodsFor: 'enumerating' stamp: 'avi 4/20/2005 14:29'!
keysDo: aBlock
	root allLeavesDo: [:ea | ea keysDo: aBlock]! !

!BTree methodsFor: 'accessing' stamp: 'avi 2/25/2004 23:32'!
order
	^ root size! !

!BTree methodsFor: 'adding' stamp: 'avi 2/26/2004 17:31'!
removeKey: aMagnitude
	| leaf |
	leaf _ root leafForKey: aMagnitude.
	leaf removeKey: aMagnitude.
	root _ leaf root! !

!BTree methodsFor: 'private' stamp: 'avi 2/23/2004 21:21'!
root
	^ root! !

!BTree methodsFor: 'accessing' stamp: 'avi 2/24/2004 23:02'!
values
	^ Array streamContents:
		[:s |
		self keysAndValuesDo: [:k :v | s nextPut: v]]! !

Collection subclass: #TreeSet
	instanceVariableNames: 'tree sortKey equalsKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:56'!
defaultEqualitySelector
	^ #=! !

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:57'!
defaultKeys
	^ BTreeKeysArray new: 64! !

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:56'!
defaultSortSelector
	^ #hash! !

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:43'!
keys: aBtreeKeys sortSelector: sortSymbol equalitySelector: equalsSymbol
	^ self basicNew initializeWithKeys: aBtreeKeys sortSelector: sortSymbol equalitySelector: equalsSymbol! !

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:55'!
new
	^ self sortBy: self defaultSortSelector! !

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:56'!
sortBy: aSymbol
	^ self sortSelector: aSymbol equalitySelector: self defaultEqualitySelector! !

!TreeSet class methodsFor: 'as yet unclassified' stamp: 'avi 5/1/2005 14:57'!
sortSelector: sortSymbol equalitySelector: equalsSymbol
	^ self keys: self defaultKeys sortSelector: sortSymbol equalitySelector: equalsSymbol! !

!TreeSet methodsFor: 'public' stamp: 'avi 5/1/2005 14:48'!
addAll: aCollection
	| oldVals newVals combined |
	(self isCompatibleTreeSetWith: aCollection) ifFalse: [^ super addAll: aCollection].

	aCollection tree keysAndValuesDo:
		[:key :vals |
		oldVals _ tree at: key ifAbsent: [#()].
		newVals _ oldVals isEmpty
					ifTrue: [vals]
					ifFalse: [vals reject: [:ea | self bucket: oldVals includes: ea]].
		newVals isEmpty ifFalse:
			[combined _ oldVals isEmpty ifTrue: [newVals] ifFalse: [oldVals, newVals].
			tree at: key put: combined]]! !

!TreeSet methodsFor: 'public' stamp: 'avi 3/2/2004 14:45'!
add: anObject
	| key bucket |
	key _ self keyForValue: anObject.
	bucket _ tree at: key ifAbsent: [#()].
	(self bucket: bucket includes: anObject) ifFalse:
		[tree at: key put: (bucket copyWith: anObject)].! !

!TreeSet methodsFor: 'public' stamp: 'ac 5/17/2005 14:07'!
atIndex: index
	| count |
	count _ 0.
	self do: [:ea | (count _ count + 1) = index ifTrue: [^ ea]].
	self error: 'No item at index: ', index asString.! !

!TreeSet methodsFor: 'private' stamp: 'avi 5/21/2004 12:18'!
bucket: anArray includes: anObject
	^ anArray anySatisfy: [:ea | (self value: anObject matches: ea)]! !

!TreeSet methodsFor: 'public' stamp: 'avi 3/2/2004 11:16'!
do: aBlock
	tree do: [:bucket | bucket do: aBlock]! !

!TreeSet methodsFor: 'public' stamp: 'avi 5/1/2005 15:50'!
equalitySelector
	^ equalsKey ifNil: [equalsKey _ self class defaultEqualitySelector]! !

!TreeSet methodsFor: 'public' stamp: 'ac 5/17/2005 14:07'!
first
	^ self atIndex: 1! !

!TreeSet methodsFor: 'public' stamp: 'avi 5/21/2004 12:30'!
includes: anObject
	| bucket |
	bucket _  tree at: (self keyForValue: anObject) ifAbsent: [^ false].
	^ self bucket: bucket includes: anObject! !

!TreeSet methodsFor: 'initialize-release' stamp: 'avi 5/1/2005 14:52'!
initializeWithKeys: aBtreeKeys sortSelector: sortSymbol equalitySelector: equalsSymbol
	self setTree: (BTree keys: aBtreeKeys) sortSelector: sortSymbol equalitySelector: equalsSymbol! !

!TreeSet methodsFor: 'public' stamp: 'avi 5/1/2005 15:28'!
intersection: aCollection
	| newTree intersection |
	(self isCompatibleTreeSetWith: aCollection) ifFalse: [^ super intersection: aCollection].

	newTree _ tree emptyCopy.
	
	tree commonKeysWith: aCollection tree keysAndValuesDo:
		[:key :left :right |
		intersection _ left select: [:ea | self bucket: right includes: ea].
		intersection isEmpty ifFalse: [newTree at: key put: intersection]].
	
	^ TreeSet basicNew setTree: newTree sortSelector: self sortSelector equalitySelector: self equalitySelector! !

!TreeSet methodsFor: 'private' stamp: 'avi 5/1/2005 15:26'!
isCompatibleTreeSetWith: aCollection
	aCollection isTreeSet ifFalse: [^ false].
	aCollection equalitySelector = self equalitySelector ifFalse: [^ false].
	aCollection sortSelector = self sortSelector ifFalse: [^ false].
	^ true! !

!TreeSet methodsFor: 'public' stamp: 'avi 5/1/2005 14:10'!
isTreeSet
	^ true! !

!TreeSet methodsFor: 'plugs' stamp: 'avi 3/2/2004 15:01'!
keyForValue: anObject
	^ anObject perform: sortKey! !

!TreeSet methodsFor: 'public' stamp: 'avi 6/13/2005 16:44'!
removeAllSuchThat: aBlock
	| changes remove |
	changes _ Dictionary new.
	tree keysAndValuesDo:
		[:key :bucket |
		remove _ bucket select: aBlock.
		remove isEmpty ifFalse:
			[changes at: key put: (bucket copyWithoutAll: remove)]].
	changes keysAndValuesDo:
		[:key :bucket |
		tree at: key put: bucket]! !

!TreeSet methodsFor: 'public' stamp: 'avi 5/21/2004 12:30'!
remove: anObject
	| key bucket |
	key _ self keyForValue: anObject.
	bucket _ tree at: key ifAbsent: [^ self].
	(self bucket: bucket includes: anObject) ifTrue:
		[bucket _ bucket reject: [:ea | self value: anObject matches: ea].
		bucket isEmpty
			ifTrue: [tree removeKey: key]
			ifFalse: [tree at: key put: bucket]]! !

!TreeSet methodsFor: 'public' stamp: 'lr 5/3/2005 17:24'!
remove: anObject ifAbsent: aBlock
	self remove: anObject.! !

!TreeSet methodsFor: 'public' stamp: 'ac 5/17/2005 14:07'!
second
	^ self atIndex: 2! !

!TreeSet methodsFor: 'initialize-release' stamp: 'avi 5/1/2005 14:51'!
setTree: aBTree sortSelector: sortSymbol equalitySelector: equalsSymbol
	tree _ aBTree.
	sortKey _ sortSymbol.
	equalsKey _ equalsSymbol! !

!TreeSet methodsFor: 'public' stamp: 'avi 4/21/2005 02:45'!
sortSelector
	^ sortKey! !

!TreeSet methodsFor: 'private' stamp: 'avi 4/20/2005 14:16'!
tree
	^ tree! !

!TreeSet methodsFor: 'private' stamp: 'avi 5/1/2005 13:55'!
tree: aBTree
	tree _ aBTree! !

!TreeSet methodsFor: 'public' stamp: 'lr 5/4/2005 16:34'!
uniqueOn: aSelector
	| result |
	result := Dictionary new.
	self do: [ :each | 
		result
			at: (each perform: aSelector)
			put: each ].
	^ result values.! !

!TreeSet methodsFor: 'plugs' stamp: 'avi 5/1/2005 15:51'!
value: anObject matches: otherObject
	^ anObject perform: self equalitySelector with: otherObject! !

Collection subclass: #TSTree
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-BTree'!

!TSTree methodsFor: 'accessing' stamp: 'avi 9/15/2004 21:11'!
at: aString
	^ self at: aString ifAbsent: [self error: aString printString, ' not found']! !

!TSTree methodsFor: 'accessing' stamp: 'avi 9/15/2004 21:11'!
at: aString ifAbsentPut: exceptionBlock
	^ self at: aString ifAbsent: [self at: aString put: exceptionBlock value]! !

!TSTree methodsFor: 'accessing' stamp: 'avi 9/22/2004 01:01'!
at: aString ifAbsent: exceptionBlock
	aString isEmpty ifTrue: [^ exceptionBlock value].
	root ifNil: [^ exceptionBlock value].
	
	^ (root lookupString: aString startingAt: 1) ifNil: exceptionBlock! !

!TSTree methodsFor: 'accessing' stamp: 'avi 9/22/2004 01:05'!
at: aString put: anObject
	aString isEmpty ifTrue: [self error: 'Keys cannot be empty'].
	root ifNil: [root _ TSTreeNode key: aString first].
	root lookupString: aString startingAt: 1 insert: anObject.
	^ anObject! !

!TSTree methodsFor: 'enumerating' stamp: 'avi 9/22/2004 01:00'!
do: aBlock
	root ifNotNil: [root do: aBlock]! !

!TSTree methodsFor: 'as yet unclassified' stamp: 'avi 9/15/2004 21:59'!
matchesForPrefix: aString
	^ Array streamContents:
		[:s |
		self matchesForPrefix: aString do:
			[:match |
			s nextPut: match]]! !

!TSTree methodsFor: 'as yet unclassified' stamp: 'avi 9/22/2004 00:57'!
matchesForPrefix: aString do: aBlock
	aString isEmpty
		ifTrue: [self do: aBlock]
		ifFalse: [root ifNotNil: [root matchesForPrefix: aString startingAt: 1 do: aBlock]]! !

!TSTree methodsFor: 'as yet unclassified' stamp: 'avi 9/15/2004 15:21'!
matchesForString: aString distance: aNumber
	^ Array streamContents:
		[:s |
		self matchesForString: aString distance: aNumber do:
			[:match |
			s nextPut: match]]! !

!TSTree methodsFor: 'as yet unclassified' stamp: 'avi 10/20/2004 21:26'!
matchesForString: aString distance: aNumber do: aBlock
	self matchesForString: aString distance: aNumber limitNodes: nil do: aBlock! !

!TSTree methodsFor: 'as yet unclassified' stamp: 'avi 10/20/2004 21:26'!
matchesForString: aString distance: aNumber limitNodes: maxNodes do: aBlock
	| nodeCount |
	nodeCount _ 0.
	aString isEmpty ifTrue: [^ self].
	root isNil ifTrue: [^ self].
	root matchesForString: aString startingAt: 1 distance: aNumber do: aBlock nodesDo:
		[:ea |
		nodeCount _ nodeCount + 1.
		nodeCount = maxNodes ifTrue: [^ self]]! !

!TSTree methodsFor: 'accessing' stamp: 'avi 12/13/2004 13:09'!
removeKey: aString
	^ self removeKey: aString ifAbsent: [self error: 'Could not find key ', aString printString]! !

!TSTree methodsFor: 'accessing' stamp: 'avi 12/13/2004 13:09'!
removeKey: aString ifAbsent: errorBlock
	| val |
	val _ root removeString: aString startingAt: 1.
	root canBeCulled ifTrue: [root _ nil].
	^ val ifNil: errorBlock! !

!TSTree methodsFor: 'accessing' stamp: 'avi 9/15/2004 21:34'!
values
	^ Array streamContents: [:s | self do: [:ea | s nextPut: ea]]! !
